##
## symlog tick helper
from matplotlib.ticker import Locator

class MinorSymLogLocator(Locator):  
  """
  Dynamically find minor tick positions based on the positions of
  major ticks for a symlog scaling.
  """
  
  def __init__(self, linthresh):
    """
    Ticks will be placed between the major ticks.
    The placement is linear for x between -linthresh and linthresh,
    otherwise its logarithmically
    """
    self.linthresh = linthresh

  def __call__(self):
    import numpy as np
  
    'Return the locations of the ticks'
    majorlocs = self.axis.get_majorticklocs()
    majorlocs = np.append(majorlocs, majorlocs[-1]*10.)
    majorlocs = np.append(majorlocs[0]*0.1, majorlocs)

    # iterate through minor locs
    minorlocs = []

    # handle the lowest part
    for i in xrange(1, len(majorlocs)):
        majorstep = majorlocs[i] - majorlocs[i-1]
        if abs(majorlocs[i-1] + majorstep/2) < self.linthresh:
            ndivs = 10
        else:
            ndivs = 9
        minorstep = majorstep / ndivs
        locs = np.arange(majorlocs[i-1], majorlocs[i], minorstep)[1:]
        minorlocs.extend(locs)

    return self.raise_if_exceeds(np.array(minorlocs))

    def tick_values(self, vmin, vmax):
        raise NotImplementedError('Cannot get tick locations for a '
			      '%s type.' % type(self))
